@@ -89,6 +89,15 @@ class SurfingEntrance(object):
         Base.metadata.create_all(bind=ViewDatabaseConnector().get_engine())
         print('done')
 
+    def init_test(self):
+        from .data.wrapper.mysql import TestDatabaseConnector
+        from .data.view.test_models import Base, FOFBase
+
+        print('Begin to create tables for database: test')
+        Base.metadata.create_all(bind=TestDatabaseConnector().get_engine())
+        FOFBase.metadata.create_all(bind=TestDatabaseConnector().get_engine())
+        print('done')
+
     def init_cas(self):
         from .data.wrapper.cas.cas_init import sync_cas_table
 
@@ -156,15 +165,24 @@ class SurfingEntrance(object):
 
     def simple_check(self):
         print('to check surfing configurator')
+        import os
         from .util.config import SurfingConfigurator
         # SurfingConfigurator is a singleton
-        _ = SurfingConfigurator(config_path='./etc/config.json')
-        print('check surfing configurator done')
+        try:
+            config_path = os.environ['SURFING_SIMPLE_CHECK_CONFIG_PATH']
+        except KeyError:
+            config_path = './etc/config.json'
+        _ = SurfingConfigurator(config_path=config_path)
+        print(f'check surfing configurator done (config_path){config_path}')
 
         print('to check raw data downloader')
         from .data.fund.raw.raw_data_downloader import RawDataDownloader
-        _ = RawDataDownloader(dry_run=True)
-        print('check raw data downloader done')
+        try:
+            dry_run = True if int(os.environ['SURFING_SIMPLE_CHECK_DRY_RUN']) else False
+        except KeyError:
+            dry_run = True
+        _ = RawDataDownloader(dry_run=dry_run)
+        print(f'check raw data downloader done (dry_run){dry_run}')
 
         print('to check basic data downloader')
         from .data.fund.basic.basic_data_processor import BasicDataProcessor
