@@ -396,7 +396,37 @@ class FOFStrategyType(ReadableEnum):
     S11 = 11                # 股票多空
     S12 = 12                # 其他（期权等）
 
+    @staticmethod
+    def get_type_of_names():
+        return {
+            '主观多头': FOFStrategyType.S1,
+            '指数增强': FOFStrategyType.S2,
+            '市场中性': FOFStrategyType.S3,
+            '管理期货': FOFStrategyType.S4,
+            '事件驱动': FOFStrategyType.S5,
+            '宏观对冲': FOFStrategyType.S6,
+            '高频套利': FOFStrategyType.S7,
+            '债券策略': FOFStrategyType.S8,
+            '组合策略': FOFStrategyType.S10,
+            '股票多空': FOFStrategyType.S11,
+            '其他（期权等）': FOFStrategyType.S12,
+        }
 
+
+FOFStrategyTypeDic = {
+    1:'主观多头',
+    2:'指数增强',
+    3:'市场中性',
+    4:'管理期货',
+    5:'事件驱动',
+    6:'宏观对冲',
+    7:'高频套利',
+    8:'债券策略',
+    10:'组合策略',
+    11:'股票多空',
+    12:'其他（期权等)',
+}
+    
 class FOFRiskType(ReadableEnum):
     """FOF产品 风险等级"""
     R1 = 1
