@@ -1401,6 +1401,105 @@ class PortfolioDataApi(metaclass=Singleton):
             print(f'failed to asset price <err_msg> {e} from port.get_asset_price_dt')
             return pd.DataFrame()
 
+    def get_product_price_part(self, begin_date, end_date, time_para, product_list: Tuple[str] = (), manager_id=None):
+        try:
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            if manager_id == None:
+                hf_info = basic.get_fof_info_strategy_exsit(fof_id_list=product_list,manager_id='1')
+            else:
+                hf_info = basic.get_fof_info_strategy_exsit(fof_id_list=product_list,manager_id=manager_id)
+
+            name_dic = hf_info.set_index('fof_id')['fof_name'].to_dict()
+            asset_info = basic.get_asset_info_real(real_id_list=product_list)
+            asset_info = asset_info.drop_duplicates('real_id')
+
+            if not asset_info.empty:
+                _name_dic = asset_info.set_index('real_id').to_dict()['real_name']
+                for k, v in _name_dic.items():
+                    name_dic[k] = v
+            _df = hf_info.rename(columns={'fof_id':'real_id','strategy_type':'asset_name','fof_name':'real_name'})[['real_id','asset_name','real_name']]
+            if not _df.empty:
+                _df.loc[:,'asset_type'] = '典型产品'
+            asset_info = asset_info.append(_df).drop_duplicates('real_id')
+            result = []
+            real_ids = product_list
+            _df1 = derived.get_fof_nav_public_adj(fof_id_list=real_ids,start_date=begin_date,end_date=end_date)
+            if not _df1.empty > 0:
+                _df1 = _df1.pivot_table(index='datetime',columns='fund_id',values='nav')
+            _df2 = derived.get_fof_nav(manager_id=manager_id, fof_id_list=real_ids)
+            if not _df2.empty > 0:
+                _df2 = _df2.pivot_table(index='datetime',columns='fof_id',values='adjusted_nav')
+                if not _df1.empty:
+                    _df = _df1.append(_df2).drop_duplicates()
+                else:
+                    _df = _df2.copy()
+            else:
+                _df = _df1.copy()
+            if not _df.empty > 0:
+                result.append(_df)
+            _df = basic.get_index_price_dt(index_list=real_ids,start_date=begin_date,end_date=end_date)
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+            df_result = pd.concat(result,axis=1).sort_index()
+            dic = {
+                'df_result':df_result, 
+                'name_dic':name_dic,
+                'asset_info':asset_info,
+            }
+            return dic
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_product_price_part')
+
+    def calculate_product_price(self, data_dic, begin_date, end_date, time_para, price_type:str='价格线', benchmark_id:str='hs300'):
+        try:
+            df_result = data_dic['df_result']
+            name_dic = data_dic['name_dic']
+            asset_info = data_dic['asset_info']
+            begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
+            if price_type == '价格线':
+                pass
+            elif price_type == '价格比线':
+                basic = BasicDataApi()
+                index_price = basic.get_index_price_dt(index_list=[benchmark_id],start_date=begin_date,end_date=end_date)
+                index_price = index_price.pivot_table(index='datetime', values='close', columns='index_id')
+                index_price = index_price.reindex(df_result.index.union(index_price.index)).sort_index().ffill().reindex(df_result.index)
+                index_price.columns = ['benchmark']
+                result = []
+                for fund_id in df_result:
+                    _df_i = df_result[[fund_id]]
+                    _df_i = _df_i.join(index_price).dropna()
+                    _df_i = _df_i / _df_i.iloc[0]
+                    _df_i[fund_id] = _df_i[fund_id] / _df_i.benchmark
+                    result.append(_df_i[[fund_id]])
+                df_result = pd.concat(result, axis=1).sort_index()
+            df_result = df_result / df_result.bfill().iloc[0]
+            df_result = df_result.rename(columns=name_dic)
+            dic = {'data':df_result,'_input_asset_nav':df_result.ffill(),'_input_asset_info':asset_info}
+            return dic
+        
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_product_price')
+
+    def get_product_price_ratio_input_fund_data(self, begin_date, end_date, time_para, fund_id):
+        try:
+            raw = RawDataApi()
+            begin_date, end_date = raw.get_date_range(time_para, begin_date, end_date)
+            hf_info = raw.get_hf_fund_info([fund_id])
+            hf_name_dic = hf_info.set_index('fund_id')['desc_name'].to_dict()
+            hf_fund_nav = raw.get_hf_fund_nav(fund_ids=[fund_id],start_date=begin_date,end_date=end_date)
+            hf_fund_nav = hf_fund_nav.pivot_table(index='datetime', values='nav', columns='fund_id')
+            data_dic = {
+                'hf_fund_nav': hf_fund_nav,
+                'hf_name_dic': hf_name_dic,
+            }
+            return data_dic
+
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.get_product_price_ratio_input_fund_data')
+
     def get_product_price(self, begin_date, end_date, time_para, product_list: Tuple[str] = (), price_type:str='价格线', benchmark_id:str='hs300'):#'价格比线'
         try:
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
@@ -1454,6 +1553,92 @@ class PortfolioDataApi(metaclass=Singleton):
         except Exception as e:
             print(f'failed to asset price <err_msg> {e} from port.get_product_price')
 
+    def product_recent_rate_data(self, product_list, manager_id=None):
+        try:
+            raw = RawDataApi()
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            if manager_id == None:
+                hf_info = basic.get_fof_info_strategy_exsit(fof_id_list=product_list,manager_id='1')
+            else:
+                hf_info = basic.get_fof_info_strategy_exsit(fof_id_list=product_list,manager_id=manager_id)
+
+            name_dic = hf_info.set_index('fof_id')['fof_name'].to_dict()
+            asset_info = basic.get_asset_info_real(real_id_list=product_list)
+            asset_info = asset_info.drop_duplicates('real_id')
+            if not asset_info.empty:
+                _name_dic = asset_info.set_index('real_id').to_dict()['real_name']
+                for k, v in _name_dic.items():
+                    name_dic[k] = v
+            result = []
+            real_ids = product_list
+            _df1 = derived.get_fof_nav_public_adj(fof_id_list=real_ids,start_date='20091231')
+            if not _df1.empty > 0:
+                _df1 = _df1.pivot_table(index='datetime',columns='fund_id',values='nav')
+            _df2 = derived.get_fof_nav(manager_id=manager_id, fof_id_list=real_ids)
+            if not _df2.empty > 0:
+                _df2 = _df2.pivot_table(index='datetime',columns='fof_id',values='adjusted_nav')
+                if not _df1.empty:
+                    _df = _df1.append(_df2).drop_duplicates()
+                else:
+                    _df = _df2.copy()
+            else:
+                _df = _df1.copy()
+            if not _df.empty > 0:
+                result.append(_df)
+            data_dic = {
+                'name_dic':name_dic,
+                'result':result,
+            }
+            return data_dic
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.product_recent_rate_data')
+    
+    def calc_product_recent_rate(self, data_dic, product_list, year:int=None, weekly:bool=False):
+        try:
+            derived = DerivedDataApi()
+            basic = BasicDataApi()
+            result = data_dic['result']
+            name_dic = data_dic['name_dic']
+            real_ids = product_list
+            _df = basic.get_index_price_dt(start_date='20091231',index_list=real_ids)
+            if not _df.empty > 0:
+                _df = _df.pivot_table(index='datetime',columns='index_id',values='close')
+                result.append(_df)
+            index_price = pd.concat(result,axis=1).sort_index().ffill().dropna()
+            if weekly:
+                # 周度
+                index_price_w = basic.data_resample_monthly_nav(index_price,rule='1W').bfill()
+                index_ret = index_price_w.pct_change(1).dropna().tail(12)
+                td = index_ret.index
+                td = [i.strftime('%Y%m%d') for i in td]
+            else:
+                if year is None:
+                    # 年度
+                    index_price_y = basic.data_resample_monthly_nav(index_price.bfill(),rule='Y')
+                    _df = index_price.iloc[[0]]
+                    index_price_y = index_price_y.append(_df).sort_index().drop_duplicates()
+                    index_ret_year = index_price_y.pct_change(1).dropna()
+                    td = index_ret_year.index
+                    td = [str(i.year) for i in td]
+                    index_ret = index_ret_year.copy()
+                else:
+                    index_price_m = index_price.loc[datetime.date(year-1,12,31):datetime.date(year,12,31)]
+                    index_price_m = basic.data_resample_monthly_nav(index_price_m,rule='1M').bfill()
+                    index_ret_m = index_price_m.pct_change(1).dropna()
+                    td = index_ret_m.index
+                    td = [str(i.year) + str(i.month).zfill(2) for i in td ]
+                    index_ret = index_ret_m.copy()
+            index_ret.index = td
+            index_ret.loc['均值',:] = index_ret.mean()
+            index_ret = index_ret.round(4)*100
+            index_ret = index_ret.rename(columns=name_dic)
+            index_ret = index_ret.T
+            index_ret.index.name = 'index_id'
+            return index_ret
+        except Exception as e:
+            print(f'failed to asset price <err_msg> {e} from port.product_recent_rate')
+
     def product_recent_rate(self, product_list, year:int=None, weekly:bool=False):
         try:
             raw = RawDataApi()
