@@ -4,4 +4,7 @@ name = 'mng_indicator_score'
 mng_s3_uri = f'{_s3_bucket_uri}/{name}.parquet'
 
 name = 'fund_indicator_score'
-fund_s3_uri = f'{_s3_bucket_uri}/{name}.parquet'
\ No newline at end of file
+fund_s3_uri = f'{_s3_bucket_uri}/{name}.parquet'
+
+name = 'mng_best_fund'
+mng_best_uri = f'{_s3_bucket_uri}/{name}.parquet'
\ No newline at end of file
