@@ -29,7 +29,7 @@ class HedgeFundDataManager(metaclass=Singleton):
         date = pd.to_datetime(date, infer_datetime_format=True).date()
 
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
-        assert part1 is not None, '!!!!'
+        assert part1 is not None, f'get pur redemp info failed (fof_id){fof_id} (manager_id){manager_id}'
         part1 = part1[part1.datetime <= date]
         part1 = part1[~part1.id.isin(exclude1)]
         part1_total_share = part1.share_changed.sum()
@@ -37,7 +37,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             part1 = part1.loc[part1.investor_id == investor_id, :]
 
         part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
-        assert part2 is not None, '!!!!'
+        assert part2 is not None, f'get div carry info failed (fof_id){fof_id} (manager_id){manager_id}'
         part2 = part2[part2.datetime <= date]
         part2 = part2[~part2.id.isin(exclude2)]
         part2_total_share = part2.share_changed.sum()
@@ -52,32 +52,50 @@ class HedgeFundDataManager(metaclass=Singleton):
         datetime: datetime.date = pd.to_datetime(datetime, infer_datetime_format=True).date()
 
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
-        assert part1 is not None, '!!!!'
+        assert part1 is not None, f'get pur redemp info failed (fof_id){fof_id} (manager_id){manager_id}'
         part1 = part1[part1.investor_id == investor_id].sort_values(by='datetime')
-        last_valid_share: float = part1[part1.datetime <= datetime].iloc[-1, :].share_after_trans
 
-        to_refresh = part1.loc[part1.datetime > datetime, :]
-        to_refresh = to_refresh.set_index('id').share_changed.cumsum() + last_valid_share
+        part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
+        assert part2 is not None, f'get div carry info failed (fof_id){fof_id} (manager_id){manager_id}'
+        part2 = part2[part2.investor_id == investor_id]
 
-        Session = sessionmaker(DerivedDatabaseConnector().get_engine())
-        db_session = Session()
-        for row in db_session.query(HedgeFundInvestorPurAndRedemp).filter(
-            HedgeFundInvestorPurAndRedemp.manager_id == manager_id,
-            HedgeFundInvestorPurAndRedemp.fof_id == fof_id,
-            HedgeFundInvestorPurAndRedemp.investor_id == investor_id,
-            HedgeFundInvestorPurAndRedemp.datetime > datetime,
-        ).all():
-            row.share_after_trans = float(to_refresh.at[row.id])
-        db_session.commit()
-        db_session.close()
+        total = pd.concat([part1, part2]).sort_values(by='datetime')
+        total = total[total.datetime <= datetime]
+        if total.empty:
+            last_valid_share: float = 0
+        else:
+            last_valid_share: float = round(total.iloc[-1, :].share_after_trans, 2)
+
+        part1_shares = part1.loc[part1.datetime > datetime, :]
+        if not part1_shares.empty:
+            part1_shares = part1_shares.set_index('datetime')
+            part2_shares = part2.loc[part2.datetime > datetime, :]
+            if not part2_shares.empty:
+                part2_shares = part2_shares.set_index('datetime')
+                part2_shares = part2_shares.reindex(part1_shares.index.union(part2_shares.index), method='ffill').reindex(part1_shares.index)
+                part1_shares['share_changed'] = part1_shares.share_changed.add(part2_shares.share_changed, fill_value=0)
+
+            to_refresh = part1_shares.set_index('id').share_changed.cumsum().round(2) + last_valid_share
+
+            Session = sessionmaker(DerivedDatabaseConnector().get_engine())
+            db_session = Session()
+            for row in db_session.query(HedgeFundInvestorPurAndRedemp).filter(
+                HedgeFundInvestorPurAndRedemp.manager_id == manager_id,
+                HedgeFundInvestorPurAndRedemp.fof_id == fof_id,
+                HedgeFundInvestorPurAndRedemp.investor_id == investor_id,
+                HedgeFundInvestorPurAndRedemp.datetime > datetime,
+            ).all():
+                row.share_after_trans = float(to_refresh.at[row.id])
+            db_session.commit()
+            db_session.close()
 
     def _calc_share_details(self, fof_info, manager_id: str, fof_id: str, investor_id: str) -> List[Dict[str, Any]]:
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
-        assert part1 is not None, '!!!!'
+        assert part1 is not None, f'get pur redemp info failed (fof_id){fof_id} (manager_id){manager_id}'
         part1 = part1.loc[part1.investor_id == investor_id, ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value']]
 
         part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
-        assert part2 is not None, '!!!!'
+        assert part2 is not None, f'get div carry info failed (fof_id){fof_id} (manager_id){manager_id}'
         part2 = part2.loc[(part2.investor_id == investor_id) & (part2.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, FOFTradeStatus.DEDUCT_REWARD])), ['datetime', 'event_type', 'share_changed', 'net_asset_value', 'acc_unit_value']]
 
         total = pd.concat([part1, part2]).sort_values(by='datetime')
@@ -88,24 +106,24 @@ class HedgeFundDataManager(metaclass=Singleton):
             elif row.event_type == FOFTradeStatus.REDEEM:
                 share_changed = abs(row.share_changed)
                 for one in share_details:
-                    assert share_changed > 0, f'!!! {share_changed}'
-                    if one['share'] >= share_changed:
+                    assert share_changed > 0, f'invalid share_changed {share_changed} (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id}'
+                    if (one['share'] > share_changed) or math.isclose(one['share'], share_changed, rel_tol=1e-6):
                         one['share'] -= share_changed
                         break
                     else:
                         share_changed -= one['share']
                         one['share'] = 0
                 else:
-                    assert False, 'not enough shares to redeem in history'
+                    assert False, f"not enough shares to redeem in history (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id} (left_shares){one['share']} (left_redemp){share_changed}"
             elif row.event_type == FOFTradeStatus.DEDUCT_REWARD:
                 for one in share_details:
-                    if math.isclose(one['share'], 0):
+                    if math.isclose(one['share'], 0, rel_tol=1e-6):
                         continue
                     if one['acc_unit_value'] >= row.acc_unit_value:
                         # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
                         continue
                     carry = round(self._calc_carry(row.acc_unit_value, one['acc_unit_value'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'] / row.net_asset_value, 2)
-                    assert one['share'] >= carry, f'!!!'
+                    assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (manager_id){manager_id} (fof_id){fof_id} (investor_id){investor_id} (left_shares){one['share']} (carry){carry}"
                     one['share'] -= carry
                     # 水位线变成本次计提时的水位线了
                     one['acc_unit_value'] = row.acc_unit_value
@@ -161,9 +179,9 @@ class HedgeFundDataManager(metaclass=Singleton):
             carry_sum = 0
             share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], datas['investor_id'])
             for one in share_details:
-                if math.isclose(one['share'], 0):
+                if math.isclose(one['share'], 0, rel_tol=1e-6):
                     continue
-                if one['share'] >= datas['redemp_share']:
+                if (one['share'] > datas['redemp_share']) or math.isclose(one['share'], datas['redemp_share'], rel_tol=1e-6):
                     one['share'] -= datas['redemp_share']
                     if datas['acc_unit_value'] > one['water_line']:
                         carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * datas['redemp_share'], 2)
@@ -174,15 +192,17 @@ class HedgeFundDataManager(metaclass=Singleton):
                         carry_sum += round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
                     one['share'] = 0
             else:
-                assert False, 'not enough shares to redeem'
+                assert False, f"not enough shares to redeem (fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (left_shares){one['share']} (left_redemp){datas['redemp_share']}"
             datas['carry_amount'] = carry_sum
             datas['redemp_confirmed_amount'] = round(datas['redemp_confirmed_amount'] - carry_sum, 2)
         else:
             raise ValueError(f'invalid event_type {event_type}')
 
         _, share_separate = self._get_total_share(datas['manager_id'], datas['fof_id'], datas['datetime'], datas['investor_id'], exclude1=[datas['id']] if 'id' in datas else [])
+
+        datas['share_changed'] = round(datas['share_changed'], 2)
         try:
-            datas['share_after_trans'] = float(share_separate.at[datas['investor_id'], 'share_changed']) + datas['share_changed']
+            datas['share_after_trans'] = round(float(share_separate.at[datas['investor_id'], 'share_changed']) + datas['share_changed'], 2)
         except KeyError:
             datas['share_after_trans'] = datas['share_changed']
 
@@ -202,8 +222,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             session.commit()
             the_id = sql_obj.id
         # df.to_sql(HedgeFundInvestorPurAndRedemp.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
-        if 'id' in datas:
-            self._refresh_datas_after_updating(datas['manager_id'], datas['fof_id'], datas['investor_id'], datas['datetime'])
+        self._refresh_datas_after_updating(datas['manager_id'], datas['fof_id'], datas['investor_id'], datas['datetime'])
         return the_id
 
     def _calc_carry(self, nav: float, water_line: float, fee_type: str, fee_desc: str) -> float:
@@ -215,7 +234,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             assert not fee_df.empty, f'invalid fee str {fee_desc} (fee_type){fee_type}'
             total_profit = 0
             for row in fee_df.itertuples(index=False):
-                assert profit > 0, '!!!!'
+                assert profit > 0, f'invalid profit (profit){profit} (nav){nav} (water_line){water_line} (fee_desc){fee_desc} (fee_type){fee_type}'
                 level_profit: float = water_line * (row.start if row.start else 0)
                 if profit > level_profit:
                     total_profit += (profit - level_profit) * (row.val if row.val else 0)
@@ -276,7 +295,8 @@ class HedgeFundDataManager(metaclass=Singleton):
         event_type = datas['event_type']
         if event_type == FOFTradeStatus.DEDUCT_REWARD:
             # 计提业绩报酬 暂不需要额外处理
-            pass
+            datas['share_changed'] = round(datas['share_changed'], 2)
+            datas['share_after_trans'] = round(datas['share_after_trans'], 2)
         else:
             total_share, share_separate = self._get_total_share(datas['manager_id'], datas['fof_id'], datas['datetime'], datas['investor_id'])
             share_separate = share_separate.share_changed.array[-1]
@@ -296,7 +316,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             else:
                 raise ValueError(f'invalid event_type {event_type}')
 
-            datas['share_after_trans'] = share_separate + datas['share_changed']
+            datas['share_after_trans'] = round(share_separate + datas['share_changed'], 2)
 
             for k in ('reinvest_amount', 'cash_dividend', 'total_dividend'):
                 datas[k] = float(datas[k])
@@ -326,7 +346,13 @@ class HedgeFundDataManager(metaclass=Singleton):
         throw excepion if failed
         '''
 
+        df = DerivedDataApi().get_hedge_fund_investor_div_carry_by_id(id)
+        assert df is not None, f'failed to get hedge fund investor data by id {id}'
+        assert df.shape[0] == 1, f'invalid id {id}'
+
+        s = df.iloc[-1, :]
         DerivedDataApi().delete_hedge_fund_investor_div_carry(id_to_delete=id)
+        self._refresh_datas_after_updating(s.manager_id, s.fof_id, s.investor_id, s.datetime)
 
     def calc_dividend_event(self, datas: Dict[str, Any]) -> pd.DataFrame:
         '''
@@ -393,7 +419,7 @@ class HedgeFundDataManager(metaclass=Singleton):
             df['cash_dividend'] = 0
         else:
             raise ValueError(f'invalid event_type {event_type}')
-        df['share_after_trans'] = share_separate + df.share_changed
+        df['share_after_trans'] = (share_separate + df.share_changed).round(2)
 
         df['fof_id'] = datas['fof_id']
         df['manager_id'] = datas['manager_id']
@@ -421,28 +447,32 @@ class HedgeFundDataManager(metaclass=Singleton):
         '''
 
         fof_info = BasicDataApi().get_fof_info(datas['manager_id'], fof_id_list=[datas['fof_id']])
-        assert fof_info is not None, '!!!!'
+        assert fof_info is not None, f"get fof info failed (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
         assert fof_info.shape[0] == 1, f"invalid fof id {[datas['fof_id']]}"
         fof_info = fof_info.iloc[-1, :]
 
         part1 = DerivedDataApi().get_hedge_fund_investor_pur_redemp(datas['manager_id'], [datas['fof_id']])
-        assert part1 is not None, '!!!!'
+        assert part1 is not None, f"get pur redemp info failed (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
 
+        part2 = DerivedDataApi().get_hedge_fund_investor_div_carry(datas['manager_id'], [datas['fof_id']])
+        assert part2 is not None, f"get pur div carry info failed (fof_id){datas['fof_id']} (manager_id){datas['manager_id']}"
+
+        total = pd.concat([part1, part2])
         carry_data = []
-        for investor_id in part1.investor_id.unique():
+        for investor_id in total.investor_id.unique():
             share_details = self._calc_share_details(fof_info, datas['manager_id'], datas['fof_id'], investor_id)
-            share_before_trans = pd.DataFrame(share_details).share.sum()
+            share_before_trans = round(pd.DataFrame(share_details).share.sum(), 2)
             carry_amount_sum = 0
             share_changed = 0
             for one in share_details:
-                if math.isclose(one['share'], 0):
+                if math.isclose(one['share'], 0, rel_tol=1e-6):
                     continue
                 if one['water_line'] >= datas['acc_unit_value']:
                     # 当前水位线未超过该笔份额的水位线 不需要计提业绩报酬
                     continue
                 carry_amount = round(self._calc_carry(datas['acc_unit_value'], one['water_line'], fof_info.incentive_fee_type, fof_info.incentive_fee_str) * one['share'], 2)
                 carry = round(carry_amount / datas['net_asset_value'], 2)
-                assert one['share'] >= carry, f'!!!'
+                assert (one['share'] > carry) or math.isclose(one['share'], carry, rel_tol=1e-6), f"not enough shares to calc carry (fof_id){datas['fof_id']} (manager_id){datas['manager_id']} (left_share){one['share']} (carry){carry}"
                 carry_amount_sum += carry_amount
                 share_changed -= carry
                 one['share'] -= carry
