@@ -110,6 +110,10 @@ class FOFNAVReader:
                     self._wechat_bot.send_hedge_and_product_nav_update(self._manager_id, r1, r2)
                 # 走到这里都认为是已经处理完了这条数据
                 uid_last_succeed = uid
+
+                abnormal_fof_ids = df[~df.fof_id.isin(self._hedge_list.union(self._product_list))]
+                if not abnormal_fof_ids.empty:
+                    raise Exception(f'got unknown fof_ids {abnormal_fof_ids.fof_id.to_list()}')
                 time.sleep(1)
             except Exception as e:
                 traceback.print_exc()
