@@ -132,85 +132,118 @@ class CalculatorBase:
     def calc_recent_ret(dates:             pd.core.series.Series,
                         values:            pd.core.series.Series,
                         frequency:         str='1D'):
-        assert frequency in ['1D'], f'frequency provided: {frequency} is not included, must in 1D'
-        dates = pd.to_datetime(dates)
-        dates = [i.date() for i in dates]
-        _end_dt = dates[-1]
-        last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
-        assert len(dates) == len(values), 'date and port_values has different numbers'
-        sr = pd.Series(values, index=dates).sort_index().dropna()
-        sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency).last().dropna()
-        sr.index = [i.date() for i in sr.index]
-        _begin_date = sr.index[0]
-        _natural_year_begin_date = datetime.date(last_date.year,1,1)
-        recent_natural_year_ret = sr.loc[:last_date][-1] / sr.loc[:max(_natural_year_begin_date,_begin_date)][-1] - 1
-        _res = {}
-        for _period, _period_begin_date in begin_date_dic.items():
-            if _period_begin_date >= _begin_date and sr.index[-1] > _period_begin_date:
-                _res[_period] = sr.loc[:last_date][-1] / sr.loc[_period_begin_date:][0] - 1
-                if _period == 'w1':
-                    _res['w1_begin_date'] = sr.loc[_period_begin_date:].index[0]
-                    _res['w1_end_date'] = sr.loc[:last_date].index[-1]
+        try:
+            assert frequency in ['1D'], f'frequency provided: {frequency} is not included, must in 1D'
+            dates = pd.to_datetime(dates)
+            dates = [i.date() for i in dates]
+            _end_dt = dates[-1]
+            last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
+            assert len(dates) == len(values), 'date and port_values has different numbers'
+            sr = pd.Series(values, index=dates).sort_index().dropna()
+            sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency).last().dropna()
+            sr.index = [i.date() for i in sr.index]
+            _begin_date = sr.index[0]
+            _natural_year_begin_date = datetime.date(last_date.year,1,1)
+            _sr = sr.loc[:_natural_year_begin_date]
+            if _sr.shape[0] > 2:
+                _natural_year_begin_date = _sr.index[-2] if _natural_year_begin_date in sr.index else _sr.index[-1]
+                recent_natural_year_ret = sr.loc[:last_date][-1] / sr.loc[_natural_year_begin_date] - 1
             else:
-                _res[_period] = np.nan
-                
-        recent_1w_ret = _res['w1']
-        recent_1m_ret = _res['m1']
-        recent_3m_ret = _res['m3']
-        recent_6m_ret = _res['m6']
-        recent_1y_ret = _res['y1']
-        recent_2y_ret = _res['y2']
-        recent_3y_ret = _res['y3']
-        recent_4y_ret = _res['y4']
-        recent_5y_ret = _res['y5']
-        if 'w1_begin_date' in _res:
-            w1_begin_date = _res['w1_begin_date']
-            w1_end_date = _res['w1_end_date']
-        else:
-            w1_begin_date = sr.index[0]
-            w1_end_date = sr.index[-1]
+                _sr = sr.loc[_natural_year_begin_date:]
+                if _sr.empty:
+                    recent_natural_year_ret = 0
+                else:
+                    _natural_year_begin_date = _sr.index[0]
+                    recent_natural_year_ret = sr.loc[:last_date][-1] / sr.loc[_natural_year_begin_date] - 1
+            _res = {}
+            for _period, _period_begin_date in begin_date_dic.items():
+                if _period_begin_date >= _begin_date and sr.index[-1] > _period_begin_date:
+                    _res[_period] = sr.loc[:last_date][-1] / sr.loc[_period_begin_date:][0] - 1
+                    if _period == 'w1':
+                        _res['w1_begin_date'] = sr.loc[_period_begin_date:].index[0]
+                        _res['w1_end_date'] = sr.loc[:last_date].index[-1]
+                else:
+                    _res[_period] = np.nan
+                    
+            recent_1w_ret = _res['w1']
+            recent_1m_ret = _res['m1']
+            recent_3m_ret = _res['m3']
+            recent_6m_ret = _res['m6']
+            recent_1y_ret = _res['y1']
+            recent_2y_ret = _res['y2']
+            recent_3y_ret = _res['y3']
+            recent_4y_ret = _res['y4']
+            recent_5y_ret = _res['y5']
+            if 'w1_begin_date' in _res:
+                w1_begin_date = _res['w1_begin_date']
+                w1_end_date = _res['w1_end_date']
+            else:
+                w1_begin_date = sr.index[0]
+                w1_end_date = sr.index[-1]
 
-        history_ret = sr.loc[:last_date][-1] / sr[0] - 1
-        days = (dates[-1] - dates[0]).days
-        history_annual_ret = math.pow(history_ret + 1, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1 if days != 0 else 0
-        sr_week = CalculatorBase.data_resample_weekly_nav(sr,rule='W-FRI')
-        if sr_week.shape[0] < 5:
-            recent_4w_ret = sr_week[-1] / sr_week[0] - 1
-        else:
-            recent_4w_ret = sr_week[-1] / sr_week[-5] - 1
+            history_ret = sr.loc[:last_date][-1] / sr[0] - 1
+            days = (dates[-1] - dates[0]).days
+            history_annual_ret = math.pow(history_ret + 1, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1 if days != 0 else 0
+            sr_week = CalculatorBase.data_resample_weekly_nav(sr,rule='W-FRI')
+            if sr_week.shape[0] < 5:
+                recent_4w_ret = sr_week[-1] / sr_week[0] - 1
+            else:
+                recent_4w_ret = sr_week[-1] / sr_week[-5] - 1
 
-        _date_20_b = datetime.date(2019,12,31)
-        _date_20_e = datetime.date(2020,12,31)
-        date_20_b = max(sr.index.min(),_date_20_b)
-        date_20_e = min(sr.index.max(),_date_20_e)
-        _sr = sr[date_20_b:date_20_e]
-        y_2020_ret = _sr[-1] / _sr[0] - 1 if not _sr.empty else None
-        lasty_year = sr.index[-1].year - 1
-        _date_last_b = datetime.date(lasty_year-1,12,31)
-        _date_last_e = datetime.date(lasty_year,12,31)
-        date_last_b = max(sr.index.min(),_date_last_b)
-        date_last_e = min(sr.index.max(),_date_last_e)
-        _sr = sr[date_last_b:date_last_e]
-        y_last_ret = _sr[-1] / _sr[0] - 1 if not _sr.empty else None
-        return {
-            'recent_natural_year_ret':recent_natural_year_ret,
-            'recent_1w_ret':recent_1w_ret,
-            'recent_4w_ret':recent_4w_ret,
-            'recent_1m_ret':recent_1m_ret,
-            'recent_3m_ret':recent_3m_ret,
-            'recent_6m_ret':recent_6m_ret,
-            'recent_1y_ret':recent_1y_ret,
-            'recent_2y_ret':recent_2y_ret,
-            'recent_3y_ret':recent_3y_ret,
-            'recent_4y_ret':recent_4y_ret,
-            'recent_5y_ret':recent_5y_ret,
-            'history_ret':history_ret,
-            'history_annual_ret':history_annual_ret,
-            'y_2020_ret':y_2020_ret,
-            'last_year_ret':y_last_ret,
-            'w1_begin_date':w1_begin_date,
-            'w1_end_date':w1_end_date,
-        }
+            _date_20_b = datetime.date(2019,12,31)
+            _date_20_e = datetime.date(2020,12,31)
+            date_20_b = max(sr.index.min(),_date_20_b)
+            date_20_e = min(sr.index.max(),_date_20_e)
+            _sr = sr[date_20_b:date_20_e]
+            y_2020_ret = _sr[-1] / _sr[0] - 1 if not _sr.empty else None
+            lasty_year = sr.index[-1].year - 1
+            _date_last_b = datetime.date(lasty_year-1,12,31)
+            _date_last_e = datetime.date(lasty_year,12,31)
+            date_last_b = max(sr.index.min(),_date_last_b)
+            date_last_e = min(sr.index.max(),_date_last_e)
+            _sr = sr[date_last_b:date_last_e]
+            y_last_ret = _sr[-1] / _sr[0] - 1 if not _sr.empty else None
+            return {
+                'recent_natural_year_ret':recent_natural_year_ret,
+                'recent_1w_ret':recent_1w_ret,
+                'recent_4w_ret':recent_4w_ret,
+                'recent_1m_ret':recent_1m_ret,
+                'recent_3m_ret':recent_3m_ret,
+                'recent_6m_ret':recent_6m_ret,
+                'recent_1y_ret':recent_1y_ret,
+                'recent_2y_ret':recent_2y_ret,
+                'recent_3y_ret':recent_3y_ret,
+                'recent_4y_ret':recent_4y_ret,
+                'recent_5y_ret':recent_5y_ret,
+                'history_ret':history_ret,
+                'history_annual_ret':history_annual_ret,
+                'y_2020_ret':y_2020_ret,
+                'last_year_ret':y_last_ret,
+                'w1_begin_date':w1_begin_date,
+                'w1_end_date':w1_end_date,
+            }
+        
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from basic.calc_recent_ret')
+            return {
+                'recent_natural_year_ret':0,
+                'recent_1w_ret':0,
+                'recent_4w_ret':0,
+                'recent_1m_ret':0,
+                'recent_3m_ret':0,
+                'recent_6m_ret':0,
+                'recent_1y_ret':0,
+                'recent_2y_ret':0,
+                'recent_3y_ret':0,
+                'recent_4y_ret':0,
+                'recent_5y_ret':0,
+                'history_ret':0,
+                'history_annual_ret':0,
+                'y_2020_ret':0,
+                'last_year_ret':0,
+                'w1_begin_date':None,
+                'w1_end_date':None,
+            }
 
     @staticmethod
     def get_period_ret(dates: pd.core.series.Series,
@@ -384,7 +417,7 @@ class CalculatorBase:
         _end_dt = dates[-1]
         last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
         assert len(dates) == len(values), 'date and port_values has different numbers'
-        if len(dates) > 2:
+        if len(dates) >= 2:
             sr = pd.Series(values, index=dates).sort_index()
             sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency).last().dropna()
             sr.index = [i.date() for i in sr.index]
